<?php

namespace Drupal\encrypt_rsa\Plugin\EncryptionMethod;

use Composer\InstalledVersions;
use Drupal\Component\Utility\NestedArray;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\encrypt\Exception\EncryptException;
use Drupal\encrypt\Plugin\EncryptionMethod\EncryptionMethodBase;
use Drupal\key_asymmetric\KeyPairInterface;
use phpseclib3\Crypt\Common\PrivateKey;
use phpseclib3\Crypt\PublicKeyLoader;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provide a private key-based RSA encryption method.
 *
 * This encryption method uses phpseclib to encrypt/decrypt using a
 * private key.
 *
 * @EncryptionMethod(
 *   id = "private_rsa",
 *   title = @Translation("Private RSA"),
 *   description = @Translation("Encrypt/Decrypt data using phpseclib."),
 *   key_type = {"pem_private"}
 * )
 */
class PrivateRsaEncryptionMethod extends EncryptionMethodBase implements ContainerFactoryPluginInterface {

  /**
   * {@inheritdoc}
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, protected KeyPairInterface $keyPair) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);

    $this->configuration = NestedArray::mergeDeep($this->defaultConfiguration(), $this->configuration);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('key_asymmetric.key_pair'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function encrypt($text, $key) {
    /** @var \phpseclib3\Crypt\RSA\PrivateKey $publicKey */
    $privateKey = PublicKeyLoader::load($key);
    // Make sure a private key has been loaded.
    if ($privateKey instanceof PrivateKey) {
      return base64_encode($privateKey->getPublicKey()->encrypt($text));
    }
    else {
      // Loaded key is not a private key.
      throw new EncryptException("The key you tried to load is not in a correct format.");
    }

  }

  /**
   * {@inheritdoc}
   */
  public function decrypt($text, $key): bool|string {
    /** @var \phpseclib3\Crypt\RSA\PrivateKey $privateKey */
    $privateKey = PublicKeyLoader::load($key);
    return $privateKey->decrypt(base64_decode($text));
  }

  /**
   * {@inheritdoc}
   */
  public function checkDependencies($text = NULL, $key = NULL) {
    $errors = [];

    if (!InstalledVersions::isInstalled('phpseclib/phpseclib')) {
      $errors[] = $this->t('phpseclib library is not loaded properly.');
    }

    if ($key) {
      $key_properties = $this->keyPair->getKeyProperties($key, '');
      if (isset($key_properties['cert'])) {
        $errors[] = $this->t('Cannot use a certificate for this encryption.');
      }
    }

    return $errors;
  }

}
