<?php

namespace Drupal\encrypt_rsa\Plugin\EncryptionMethod;

use Composer\InstalledVersions;
use Drupal\Component\Utility\NestedArray;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\encrypt\Exception\EncryptException;
use Drupal\encrypt\Exception\EncryptionMethodCanNotDecryptException;
use Drupal\encrypt\Plugin\EncryptionMethod\EncryptionMethodBase;
use Drupal\key_asymmetric\KeyPairInterface;
use phpseclib3\Crypt\Common\PublicKey;
use phpseclib3\Crypt\PublicKeyLoader;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provide a public key-based RSA encryption method.
 *
 * This encryption method uses phpseclib to only encrypt using
 * a public key.
 *
 * @EncryptionMethod(
 *   id = "public_rsa",
 *   title = @Translation("Public RSA"),
 *   description = @Translation("Encrypt(-only) data using phpseclib."),
 *   key_type = {"pem_public"},
 *   can_decrypt = FALSE
 * )
 */
class PublicRsaEncryptionMethod extends EncryptionMethodBase implements ContainerFactoryPluginInterface {

  /**
   * {@inheritdoc}
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, protected KeyPairInterface $keyPair) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);

    $this->configuration = NestedArray::mergeDeep($this->defaultConfiguration(), $this->configuration);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('key_asymmetric.key_pair'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function encrypt($text, $key) {
    /** @var \phpseclib3\Crypt\RSA\PublicKey $publicKey */
    $publicKey = PublicKeyLoader::load($key);
    // Make sure a public key has been loaded.
    if ($publicKey instanceof PublicKey) {
      return base64_encode($publicKey->encrypt($text));
    }
    else {
      // Loaded key is not a public key.
      throw new EncryptException("The key you tried to load is not in a correct format.");
    }
  }

  /**
   * {@inheritdoc}
   */
  public function decrypt($text, $key) {
    throw new EncryptionMethodCanNotDecryptException("Decrypting with a public key is not possible.");
  }

  /**
   * {@inheritdoc}
   */
  public function checkDependencies($text = NULL, $key = NULL) {
    $errors = [];

    if (!InstalledVersions::isInstalled('phpseclib/phpseclib')) {
      $errors[] = $this->t('phpseclib library is not loaded properly.');
    }

    if ($key) {
      $key_properties = $this->keyPair->getKeyProperties($key, '');
      if (isset($key_properties['cert'])) {
        $errors[] = $this->t('Cannot use a certificate for this encryption.');
      }
    }

    return $errors;
  }

}
